package com.tr.tms.abstractionLayer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class CalaisObject {
    /**
     * URI of the the object
     */
    private String objectId;

    /**
     * The type of the object for example:"http://.../1/type/em/r/Merger"
     */
    private String type;

    private Map<String, List<String>> literals = new HashMap<>();//key=field name, value=list of field values
    private Map<String, List<String>> externalURIs = new HashMap<>();//key=field name, value=list of URI values
    private Map<String, List<CalaisObject>> references = new HashMap<>();//key=field name, value=list of field values CalaisObject
    private Map<String, Map<String, List<CalaisObject>>> backReferences = new HashMap<>();//key=field name value = map: key=type, value=list of CalaisObject

    public CalaisObject(String objectId, String type) {
        this.objectId = objectId;
        this.type = type;
    }

    public String getObjectId() {
        return objectId;
    }

    public String getType() {
        return type;
    }

    public Map<String, List<String>> getLiterals() {
        return literals;
    }

    public Map<String, List<CalaisObject>> getReferences() {
        return references;
    }

    public Map<String, Map<String, List<CalaisObject>>> getBackReferences() {
        return backReferences;
    }

    public Map<String, List<CalaisObject>> getBackReferencesByFieldName(String fieldName) {
        return backReferences.get(fieldName);
    }

    public Map<String, List<String>> getExternalURIs() {
        return externalURIs;
    }

    public void addLiteral(String fieldName, String value){
        List<String> values = literals.get(fieldName);
        if(values == null){
            values = new ArrayList<>();
            literals.put(fieldName, values);
        }
        values.add(value);
    }

    public void addReference(String fieldName, CalaisObject value){
        List<CalaisObject> values = references.get(fieldName);
        if(values == null){
            values = new ArrayList<>();
            references.put(fieldName, values);
        }
        values.add(value);
    }

    public void addBackReference(String fieldName, CalaisObject calaisObject){
        final String type = calaisObject.getType();
        Map<String, List<CalaisObject>> typesValues = backReferences.get(fieldName);
        if(typesValues == null){
            typesValues = new HashMap<String, List<CalaisObject>>();
            backReferences.put(fieldName, typesValues);
        }
        List<CalaisObject> values = typesValues.get(type);
        if(values == null){
            values = new ArrayList<>();
            typesValues.put(type, values);
        }
        values.add(calaisObject);
    }

    public void addExternalURI(String fieldName, String value){
        List<String> values = externalURIs.get(fieldName);
        if(values == null){
            values = new ArrayList<>();
            externalURIs.put(fieldName, values);
        }
        values.add(value);
    }



}
